<?php

/**
 * Database access function(s)
 *
 * @copyright    CONTREXX CMS - COMVATION AG
 * @author        Comvation Development Team <info@comvation.com>
 * @package     contrexx
 * @subpackage  core
 * @version        1.0.0
 */

/**
 * @ignore
 */
require_once ASCMS_LIBRARY_PATH.'/adodb/adodb.inc.php';

/**
 * Factory callback for AdoDB NewConnection
 * @deprecated Use Doctrine!
 * @return \Cx\Core\Db\CustomAdodbPdo 
 */
function cxAdodbPdoConnectionFactory() {
    global $pdo;
    
    $obj = new \Cx\Core\Db\CustomAdodbPdo($pdo);
    return $obj;
}

/**
 * Returns the database object.
 *
 * If none was created before, or if {link $newInstance} is true,
 * creates a new database object first.
 * In case of an error, the reference argument $errorMsg is set
 * to the error message.
 * @author  Comvation Development Team <info@comvation.com>
 * @deprecated Use Doctrine instead
 * @access  public
 * @version 1.0.0
 * @param   string  $errorMsg       Error message
 * @param   boolean $newInstance    Force new instance
 * @global  array                   Language array
 * @global  array                   Database configuration
 * @global  integer                 ADODB fetch mode
 * @return  boolean                 True on success, false on failure
 */
function getDatabaseObject(&$errorMsg, $newInstance = false)
{
    global $_DBCONFIG, $ADODB_NEWCONNECTION, $ADODB_FETCH_MODE, $_CONFIG, $pdo;
    static $objDatabase;

    if (is_object($objDatabase) && !$newInstance) {
        return $objDatabase;
    } else {
        if (!isset($pdo)) {
            $objDateTimeZone = new DateTimeZone($_CONFIG['timezone']);
            $objDateTime = new DateTime('now', $objDateTimeZone);
            $offset = $objDateTimeZone->getOffset($objDateTime);
            $offsetHours = round(abs($offset)/3600); 
            $offsetMinutes = round((abs($offset)-$offsetHours*3600) / 60); 
            $offsetString = ($offset > 0 ? '+' : '-').($offsetHours < 10 ? '0' : '').$offsetHours.':'.($offsetMinutes < 10 ? '0' : '').$offsetMinutes;

            $pdo = new \PDO(
                'mysql:dbname=' . $_DBCONFIG['database'] . ';charset=' . $_DBCONFIG['charset'] . ';host='.$_DBCONFIG['host'],
                $_DBCONFIG['user'],
                $_DBCONFIG['password'],
                array(
                    // Setting the connection character set in the DSN (see below new \PDO()) prior to PHP 5.3.6 did not work.
                    // We will have to manually do it by executing the SET NAMES query when connection to the database.
                    \PDO::MYSQL_ATTR_INIT_COMMAND => 'SET NAMES '.$_DBCONFIG['charset'],
                    \PDO::MYSQL_ATTR_INIT_COMMAND => 'SET time_zone = \'' . $offsetString . '\'',
                )
            );
            $pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_SILENT);
        }
        
        // open db connection
        \Env::get('ClassLoader')->loadFile(ASCMS_LIBRARY_PATH.'/adodb/adodb.inc.php');
        $ADODB_FETCH_MODE = ADODB_FETCH_ASSOC;
        $ADODB_NEWCONNECTION = 'cxAdodbPdoConnectionFactory';
        $objDb = \ADONewConnection('pdo');

        $errorNo = $objDb->ErrorNo();
        if ($errorNo != 0) {
            if ($errorNo == 1049) {
                $errorMsg = 'The database is unavailable';
            } else {
                $errorMsg =  $objDb->ErrorMsg()."<br />";
            }
            unset($objDb);
            return false;
        }
        if ($newInstance) {
            return $objDb;
        } else {
            $objDatabase = $objDb;
            return $objDb;
        }
    }
}

?>
